//	PhilComponent.h - The Phil file manager/browser Component.
//	--------------------------------------------------------------------------
//	This file is part of 'Phil', a very simple file manager/browser for the
//	JUCE library.
//	Copyright (C) 2005-2006  Niall Moody
//	
//	This program is free software; you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation; either version 2 of the License, or
//	(at your option) any later version.
//	
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//	
//	You should have received a copy of the GNU General Public License
//	along with this program; if not, write to the Free Software
//	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//	
//	--------------------------------------------------------------------------

#ifndef PHILCOMPONENT_H_
#define PHILCOMPONENT_H_

/*!
	\mainpage

	Phil is a fairly simplistic Component to view/manipulate files for JUCE.
	I made it because I needed a FileChooser for my Twindy window manager for
	linux, and (as of the time of writing) JUCE doesn't have one on linux yet.
	I figured a file view Component could be useful outside a file chooser
	dialog, so I tried to make it as flexible as possible.  As such there's
	a simplistic file browser application (PhilBrowser) included, based on
	the Phil Component.

	The main Component is Phil, which keeps track of all the relevant
	information like which directory we're currently viewing, clipboard
	contents etc.  Phil displays the contents of the current directory using
	a FileView subclass.  At present there's just one of these: ListFileView,
	but it should be relatively easy to implement others.  In addition, 
	Phil supports additional Components to extend it's functionality: a
	toolbar, sidebar and statusbar Component.  There are three of these
	Components included at present: the StandardPhilToolbar,
	StandardPhilStatusbar, and the PhilChooserStatusbar.  The
	StandardPhilToolbar just provides 'up' and 'home' buttons, as well as a
	basic 'back'/'forward' history; StandardPhilStatusbar displays some
	information about the currently-displayed directory, and the
	currently-selected file, and PhilChooserStatusbar has a number of
	Components common to file chooser dialogs.

	Also included is a ExternalProgram class, used to launch external
	programs - it is cross-platform compatible and will work on Windows,
	OS X and Linux.
 */

class FileView;

///	The Phil file manager/browser Component.
/*!
	Phil is a ChangeBroadcaster, so you can be informed when the user has
	selected a new file/directory.  This is particularly useful if you've
	made your own sidebar or statusbar, and want to display some information
	about the currently-selected file.
 */
class Phil : public Component,
			 public ChangeBroadcaster,
			 public ApplicationCommandTarget
{
  public:
	///	Constructor.
	Phil();
	///	Destructor.
	~Phil();

	///	Draws the Component.
	void paint(Graphics& g);

	///	Places the various child Components in their correct positions.
	void resized();

	///	Returns the current viewed directory.
	/*!
		Note that this may be File::nonexistent, if our current position is
		at the file system's root (i.e. on Windows we'll be able to see
		what disks we have (c:, d:, etc.), while on linux/OSX, it'll just
		be / ).  To be safe, either check that the return value points to a
		valid path, or only use methods from this class, which should be safe.
	 */
	const File& getCurrentDirectory() const;
	///	Sets the current viewed directory.
	/*!
		\param dir The directory to change to.
		\param passOnChangeMessage If true, Phil will call
		ChangeBroadcaster::sendChangeMessage(), to update any ChangeListeners.
	 */
	void setCurrentDirectory(const File& dir, bool passOnChangeMessage = true);
	///	Returns the full path names of the currently-highlghted file(s).
	/*!
		Multiple files may be selected.  Be sure to check the size of the
		array, in case there aren't any files currently highlighted.
	 */
	const StringArray& getCurrentlyHighlightedFiles() const;

	///	Sets whether or not the user may select multiple files.
	/*!
		This is true by default.
	 */
	void setMultipleFilesSelected(bool val = false);
	///	Called from ListFileView when a file has been selected.
	void fileSelected(long index);
	///	Called from ListFileView for multiple files selected.
	/*!
		How this works is, on mouse down, ListFileView calls fileSelected()
		for the file under the cursor.  If the user then holds the button
		down, and drags (while holding ctrl/command), to select more files,
		this method gets called for each file.
	 */
	void anotherFileSelected(long index);
	///	Called to select all objects in the current directory.
	/*!
		\param val True to select all objects, false to deselect all objects.
	 */
	void selectAll(bool val);
	///	Called (from ListFileView) to display a context menu.
	/*!
		\param file Index of the file/directory that was clicked on.
		\param x X position to display the menu at.
		\param y Y position to display the menu at.
	 */
	void displayContextMenuNow(long file, int x, int y);

	///	Call this to mask out files which don't have an extension in wildcardPattern.
	/*!
		\param wildcardPattern StringArray holding an entry for each extension
		(e.g. to show .exe and .txt files, it would be: wildcarPattern[0] = 
		T("exe"); wildcardPattern[1] = T("txt");
		\param displayAll If true, all files are displayed, otherwise only
		those with extensions matching those in wildcardPattern are displayed.
	 */
	void maskExtensions(const StringArray& wildcardPattern,
						bool displayAll = false);
	///	Called to set which extension in our list of visible extensions to display.
	/*!
		\param index Index to the visibleExtensions array we hold (find out
		what each extension is by calling getExtensions().

		If index is not within the valid range of indices, all file extensions
		will be displayed.
	 */
	void setVisibleExtension(long index);
	///	Returns the current list of visible extensions we may display.
	const StringArray& getVisibleExtensions() const {return visibleExtensions;};

	///	Used to get the child files of the currentDirectory.
	/*!
		Basically the same as File::findChildFiles(), but takes into account
		how visibleExtensions is set.
	 */
	int findChildFiles(OwnedArray<File> &results,
					   const bool searchForDirectories);
	///	Used to get the number of child files in the current directory.
	/*!
		Basically the same as File::getNumberOfChildFiles(), but takes into
		account how visibleExtensions is set.
	 */
	int getNumberOfChildFiles(const bool searchForDirectories);

	///	Called when the user wants to copy the selected object(s).
	void copyObject();
	///	Called when the user wants to cut the selected object(s).
	void cutObject();
	///	Called when the user wants to paste what's on the clipboard.
	void pasteObject();
	///	Called when the user wants to select all objects in the current directory.
	void selectAllObjects();
	///	Called when the user wants to delete the currently-selected object.
	void deleteObject();
	///	Called when the user wants to move to the parent directory.
	void navToParent();
	///	Called whent the user wants to move the cursor up.
	/*!
		\param multiple True if the user is holding down shift, and wants to
		select multiple files.
	 */
	void cursorUp(bool multiple);
	///	Called whent the user wants to move the cursor down.
	/*!
		\param multiple True if the user is holding down shift, and wants to
		select multiple files.
	 */
	void cursorDown(bool multiple);
	///	Called whent the user wants to move the cursor left.
	/*!
		\todo ****NOT IMPLEMENTED****
	 */
	void cursorLeft();
	///	Called whent the user wants to move the cursor right.
	/*!
		\todo ****NOT IMPLEMENTED****
	 */
	void cursorRight();
	///	Called when the user wants to 'enter' an object.
	/*!
		If the object is a directory, this will cause the browser to enter it,
		if it's a file, the browser will assume it's an executably and try
		to run it using ExternalProgram::run() (unless PHIL_LIGHT is defined).
	 */
	void enterObject();
	///	Called when the user wants to open an object with a particular program.
	/*!
		Uses ExternalProgram::run() to run the program (assuming PHIL_LIGHT
		is not defined).
	 */
	void openWith();
	///	Called when the user wants to inspect the properties of the selected file/directory.
	/*!
		Does nothing if PHIL_LIGHT is defined.
	 */
	void displayProperties();
	///	Creates a new directory in currentDirectory.
	void createNewDirectory();
	///	Renames the currently-selected file or directory.
	void renameObject();

	///	Returns the current contents of the clipboard.
	const StringArray& getClipboard() const;
	///	Sets the contents of the clipboard.
	void setClipboardContents(const StringArray& newContents);
	///	Returns whether or not the current clipboard operation is cut.
	bool getIsCut() const {return isCut;};
	///	Sets whether or not the current clipboard operation is cut.
	/*!
		\param val True = cut, false = copy.
	 */
	void setIsCut(bool val);

	///	Set whether or not to display the context menu when displayContextMenuNow() is called.
	/*!
		\sa displayContextMenu
	 */
	void setDisplayContextMenu(bool val);
	///	Set whether or not to run external programs from Phil.
	/*!
		\sa allowExternalPrograms
	 */
	void setAllowExternalPrograms(bool val);
	///	Whether or not to allow drag and drop.
	/*!
		\param val true = allow it, false = don't allow it.
	 */
	void setAllowDragAndDrop(bool val);
	///	Returns whether or not to allow drag and drop.
	bool getDragAndDropAllowed() const {return allowDragAndDrop;};

	///	Returns a pointer to the current FileView.
	FileView *getFileView() const {return fileView;};
	///	Sets the FileView for Phil to use.
	/*!
		Phil will delete this itself.  Note that Phil will use a ListFileView
		by default.
	 */
	void setFileView(FileView *newView);

	///	Call this to set a Component to act as a toolbar.
	/*!
		The toolbar Component is displayed above the main file view.  You can
		set the height of the toolbar with the setToolbarHeight() method.

		Phil is responsible for deleting the toolbar Component.
	 */
	void setToolbarComponent(Component *component);
	///	Call this to set the height of the toolbar Component.
	/*!
		The default height is 32 pixels.
	 */
	void setToolbarHeight(int val);
	///	Call this to set a Component to act as a sidebar.
	/*!
		The sidebar Component is displayed to the left of the main file view.
		You can set the width of the sidebar with the setSidebarWidth()
		method.

		Phil is responsible for deleting the sidebar Component.
	 */
	void setSidebarComponent(Component *component);
	///	Call this to set the width of the sidebar Component.
	/*!
		The default width is 128 pixels.
	 */
	void setSidebarWidth(int val);
	///	Call this to set a Component to act as a statusbar.
	/*!
		The statusbar Component is displayed below the main file view.  You
		can set the height of the toolbar with the setStatusbarHeight()
		method.

		Phil is responsible for deleting the statusbar Component.
	 */
	void setStatusbarComponent(Component *component);
	///	Call this to set the height of the statusbar Component.
	/*!
		The default height is 24 pixels.
	 */
	void setStatusbarHeight(int val);

	///	Returns the toolbar Component.
	Component *getToolbarComponent() const {return toolbarComponent;};
	///	Returns the height of the toolbar Component.
	int getToolbarHeight() const {return toolbarHeight;};
	///	Returns the sidebar Component.
	Component *getSidebarComponent() const {return sidebarComponent;};
	///	Returns the width of the sidebar Component.
	int getSidebarWidth() const {return sidebarWidth;};
	///	Returns the statusbar Component.
	Component *getStatusbarComponent() const {return statusbarComponent;};
	///	Returns the height of the statusbar Component.
	int getStatusbarHeight() const {return statusbarHeight;};

	///	Returns the colour used for the background.
	const Colour& getBackgroundColour() const;
	///	Returns the colour used for the boundary.
	const Colour& getBoundaryColour() const;
	///	Returns the colour used to highlight selected files.
	const Colour& getHighlightColour() const;
	///	Returns the colour used for file names.
	const Colour& getTextColour() const;
	///	Sets the various colours.
	void setColours(const Colour& background,
					const Colour& boundary = Colours::lightgrey,
					const Colour& text = Colours::grey,
					const Colour& highlight = Colours::lightblue);

	///	Callback used to act on keyboard shortcuts.
	//static void shortcutCallback(const KeyPressCallbackFunctionInfo &keyPressInfo);

	///	So we can handle keyboard shortcuts.
	ApplicationCommandTarget *getNextCommandTarget() {return 0;};
	///	So we can handle keyboard shortcuts.
	void getAllCommands(Array<CommandID>& commands);
	///	So we can handle keyboard shortcuts.
	void getCommandInfo(const CommandID commandID,
						ApplicationCommandInfo& result);
	///	So we can handle keyboard shortcuts.
	bool perform(const InvocationInfo& info);
	///	So the top-level window can register appCommands as a key listener.
	KeyPressMappingSet *getKeyMappings() {return appCommands.getKeyMappings();};

	///	Enum holding the various commandUIDs used in the shortcutCallback.
	enum
	{
		editCopy=1,	//<	Copy the selected file.
		editCut,	//<	Cut the selected file.
		editPaste,	//<	Paste the file currently on the clipboard.
		editSelect,	//<	Select/deselect all objects in the current directory.
		editDeselect,//<	Deselect all objects in the current directory.
		editDelete,	//<	Delete the selected file.
		editRename,	//<	Rename the selected file/directory.
		editNewDir,	//<	Create a new directory in currentDirectory.
		navParent,	//<	Move to the parent directory.
		navUp,		//<	Select the object above the currently-selected one.
		navSelectUp,
		navDown,	//<	Select the object below the currently-selected one.
		navSelectDown,
		navLeft,	//<	Select the object to the left of the currently-selected one.
		navRight,	//<	Select the object to the right of the currently-selected one.
		navReturn,	//<	Enter/run the currently-selected directory/file.
		navSpace,	//<	Open the selected file/directory with a particular program.
		navProperties//<	Display properties for the selected file/directory.
	};

	juce_UseDebuggingNewOperator
  private:
	///	Helper method.  Called in constructor to set up shortcuts.
	void setupShortcuts();
	///	Component displaying our current directory.
	FileView *fileView;
	///	Viewport to hold fileView.
	Viewport *view;
	//	Used to handle handle shortcuts (copy, paste etc.).
	/*!
		Also handles directory navigation (left, right etc.).
	 */
	//KeyPressMappingSet shortcuts;

	///	The current directory we are viewing.
	File currentDirectory;
	///	Array holding any currently-selected files.
	OwnedArray<File> selectedFiles;
	///	Whether or not multiple files may be selected.
	bool multipleFilesSelected;
	///	Array holding the contents of the clipboard.
	StringArray clipboardContents;
	///	Whether or not a paste operation should move or copy the clipboard contents.
	bool isCut;
	///	StringArray holding extensions which are not masked.
	/*!
		If the Component is set to mask out certain extensions, the allowed
		extensions will be in here.  Otherwise it will be empty.
	 */
	StringArray visibleExtensions;
	///	The index to the visibleExtensions entry that is currently displayed.
	long currentVisibleExtensions;

	///	Whether or not we want to display the context menu or not.
	/*!
		True by default.

		This was originally intended for PhilChooser, but I'm not sure if it's
		necessary in that case to disallow context menu operations (delete
		and new folder can still be useful in that situation for example).
	 */
	bool displayContextMenu;
	///	Whether or not to allow external programs to be run.
	/*!
		True by default.

		This is intended for PhilChooser, where it doesn't make sense to allow
		the 'Run' and 'Open with..' operations.
	 */
	bool allowExternalPrograms;
	///	Whether or not to allow drag and drop to move files.
	bool allowDragAndDrop;

	///	The bounds of the file view.
	Rectangle fileViewBounds;

	///	Colour of the file view boundary.
	Colour fileViewBoundaryColour;
	///	Colour of the file view background.
	Colour fileViewBackgroundColour;
	///	Colour of the file view text.
	Colour fileViewTextColour;
	///	Colour of highlighted items in the file view.
	Colour fileViewHighlightColour;

	///	The toolbar Component.
	Component *toolbarComponent;
	///	The height of the toolbar Component.
	int toolbarHeight;
	///	The sidebar Component.
	Component *sidebarComponent;
	///	The width of the sidebar Component.
	int sidebarWidth;
	///	The statusbar Component.
	Component *statusbarComponent;
	///	The height of the statusbar Component.
	int statusbarHeight;

	///	Used to handle the key bindings.
	ApplicationCommandManager appCommands;
};

//----------------------------------------------------------------------------
///	ElementComparator for sorting Strings.
/*!
	On Linux, File::findChildFiles() returns the files sorted by date(?),
	which isn't particularly helpful for us, so we have to sort them
	alphabetically with this.
 */
template <class ElementType>
class StringComparator  : public ElementComparator<ElementType>
{
    int compareElements (ElementType first,
                         ElementType second)
    {
        return first->getFileName().compare(second->getFileName());
    }
};

#endif
